mod common;
mod tui;
mod cli;

use bpaf::{construct, Parser};
use crate::common::{config, repository, update};

#[derive(Debug, Clone)]
enum Commands {
    Tui(),
    Cli(cli::CliCommand)
}

fn main() {
    let tui_cmd =
        bpaf::command("tui", construct!(Commands::Tui()).to_options())
            .help("Terminal ui for interactive use");

    let cli_parser = cli::get_parser();

    let cli_cmd =
        bpaf::command("cli", construct!(Commands::Cli(cli_parser)).to_options())
            .help("Command line for script usage");

    let opt = construct!([tui_cmd, cli_cmd])
        .fallback(Commands::Tui())
        .to_options()
        .descr("Dotfiles multitool")
        .run();

    if let (Some(version), Some(name)) = (option_env!("BUILD_ID"), option_env!("JOB_BASE_NAME")) {
        println!("Starting installer version {name}-{version}");
        update::check_for_updates(version.parse().unwrap(), name);
    } else {
        println!("Starting installer version unknown");
    }

    if !repository::open_repo() {
        println!("Failed to open/clone repo!");
        return;
    }

    if !std::path::Path::new("repo/mod.toml").exists() {
        println!("'mod.toml' doesn't exist!");
        return;
    }

    let mods = match config::parse() {
        Ok(v) => v,
        Err(err) => {
            println!("Failed to process mod.toml!");
            println!("{}", err);
            return;
        }
    };

    match opt {
        Commands::Tui() => tui::run(&mods),
        Commands::Cli(command) => cli::run(&mods, command)
    }
}
